"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultPropertySortFunction = exports.defaultGenerateIdentifierType = exports.defaultGetPropertyMetadata = exports.defaultGetMetadata = void 0;
const recase_1 = require("@kristiandupont/recase");
const path_1 = require("path");
const tagged_comment_parser_1 = require("tagged-comment-parser");
const resolveType_1 = __importDefault(require("./generators/resolveType"));
const toPascalCase = (0, recase_1.recase)(null, "pascal");
// #region defaultGetMetadata
const defaultGetMetadata = (details, generateFor, instantiatedConfig) => {
    const { comment: strippedComment } = (0, tagged_comment_parser_1.tryParse)(details.comment);
    const isAgentNoun = ["initializer", "mutator"].includes(generateFor);
    const relationComment = isAgentNoun
        ? `Represents the ${generateFor} for the ${details.kind} ${details.schemaName}.${details.name}`
        : `Represents the ${details.kind} ${details.schemaName}.${details.name}`;
    const suffix = isAgentNoun ? `_${generateFor}` : "";
    return {
        name: toPascalCase(details.name + suffix),
        comment: [relationComment, ...(strippedComment ? [strippedComment] : [])],
        path: (0, path_1.join)(instantiatedConfig.outputPath, details.schemaName, toPascalCase(details.name)),
    };
};
exports.defaultGetMetadata = defaultGetMetadata;
// #endregion defaultGetMetadata
// #region defaultGetPropertyMetadata
const defaultGetPropertyMetadata = (property, _details, generateFor, _instantiatedConfig) => {
    const { comment: strippedComment } = (0, tagged_comment_parser_1.tryParse)(property.comment);
    return {
        name: property.name,
        comment: [
            ...(strippedComment ? [strippedComment] : []),
            ...(generateFor === "initializer" && property.defaultValue
                ? [`Default value: ${property.defaultValue}`]
                : []),
        ],
    };
};
exports.defaultGetPropertyMetadata = defaultGetPropertyMetadata;
// #endregion defaultGetPropertyMetadata
// #region defaultGenerateIdentifierType
const defaultGenerateIdentifierType = (column, details, config) => {
    const name = toPascalCase(details.name) + toPascalCase(column.name);
    const innerType = (0, resolveType_1.default)(column, details, {
        ...config,
        // Explicitly disable identifier resolution so we get the actual inner type here
        generateIdentifierType: undefined,
    });
    const imports = [];
    let type = innerType;
    if (typeof innerType === "object") {
        // Handle non-primitives
        type = innerType.name;
        imports.push(...innerType.typeImports);
    }
    return {
        declarationType: "typeDeclaration",
        name,
        exportAs: "named",
        typeDefinition: [`${type} & { __brand: '${name}' }`],
        typeImports: imports,
        comment: [`Identifier type for ${details.schemaName}.${details.name}`],
    };
};
exports.defaultGenerateIdentifierType = defaultGenerateIdentifierType;
// #endregion defaultGenerateIdentifierType
// #region defaultPropertySortFunction
const defaultPropertySortFunction = (a, b) => {
    if (a.isPrimaryKey && !b.isPrimaryKey) {
        return -1;
    }
    else if (!a.isPrimaryKey &&
        b.isPrimaryKey) {
        return 1;
    }
    else {
        // return a.name.localeCompare(b.name);
        return a.ordinalPosition - b.ordinalPosition;
    }
};
exports.defaultPropertySortFunction = defaultPropertySortFunction;
// #endregion defaultPropertySortFunction
