"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const tagged_comment_parser_1 = require("tagged-comment-parser");
const generateProperties_1 = __importDefault(require("./generateProperties"));
const makeMapper = (config) => (details) => {
    if (details.kind === "compositeType") {
        // If a composite type has a @type tag in the comment,
        // we will use that type instead of a generated one.
        const { tags } = (0, tagged_comment_parser_1.tryParse)(details.comment);
        if (tags?.type) {
            return [];
        }
    }
    const declarations = [];
    const { name: selectorName, comment: selectorComment, path, } = config.getMetadata(details, "selector", config);
    if (details.kind === "table" && config.generateIdentifierType) {
        const { columns } = details;
        const identifierColumns = columns.filter((c) => c.isPrimaryKey && !c.reference);
        identifierColumns.forEach((c) => declarations.push(config.generateIdentifierType(c, details, config)));
    }
    const selectorProperties = (0, generateProperties_1.default)(details, "selector", config);
    const selectorDeclaration = {
        declarationType: "interface",
        name: selectorName,
        comment: selectorComment,
        exportAs: "default",
        properties: selectorProperties,
    };
    declarations.push(selectorDeclaration);
    if (details.kind === "table") {
        const { name: initializerName, comment: initializerComment } = config.getMetadata(details, "initializer", config);
        const initializerProperties = (0, generateProperties_1.default)(details, "initializer", config);
        const initializerDeclaration = {
            declarationType: "interface",
            name: initializerName,
            comment: initializerComment,
            exportAs: "named",
            properties: initializerProperties,
        };
        declarations.push(initializerDeclaration);
        const { name: mutatorName, comment: mutatorComment } = config.getMetadata(details, "mutator", config);
        const mutatorProperties = (0, generateProperties_1.default)(details, "mutator", config);
        const mutatorDeclaration = {
            declarationType: "interface",
            name: mutatorName,
            comment: mutatorComment,
            exportAs: "named",
            properties: mutatorProperties,
        };
        declarations.push(mutatorDeclaration);
    }
    return declarations.map((declaration) => ({ path, declaration }));
};
// "Composite" in this case means tables, views, materialized views and composite types.
// I.e. anything that has "properties" and will be turned into an interface in Typescript.
const makeCompositeGenerator = (kind, config) => (schema, outputAcc) => {
    const mapper = makeMapper(config);
    const declarations = schema[`${kind}s`]?.map(mapper).flat() ?? [];
    return declarations.reduce((acc, { path, declaration }) => {
        const existing = acc[path];
        if (existing) {
            existing.declarations.push(declaration);
        }
        else {
            acc[path] = {
                declarations: [declaration],
            };
        }
        return acc;
    }, outputAcc);
};
exports.default = makeCompositeGenerator;
