"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tagged_comment_parser_1 = require("tagged-comment-parser");
const makeMapper = (config) => (domainDetails) => {
    // If a domain has a @type tag in the comment,
    // we will use that type instead of a generated one.
    const { tags } = (0, tagged_comment_parser_1.tryParse)(domainDetails.comment);
    if (tags?.type) {
        return undefined;
    }
    const { name, comment, path } = config.getMetadata(domainDetails, undefined, config);
    let typeDefinition = [];
    const typeImports = [];
    const mapped = config.typeMap[domainDetails.innerType];
    if (!mapped) {
        typeDefinition = ["unknown"];
        console.warn(`Domain '${name}' has unknown type '${domainDetails.innerType}'`);
    }
    else if (typeof mapped === "string") {
        typeDefinition = [mapped];
    }
    else {
        typeDefinition = [mapped.name];
        typeImports.push(...mapped.typeImports);
    }
    const declaration = {
        declarationType: "typeDeclaration",
        name,
        comment,
        exportAs: "default",
        typeDefinition,
        typeImports,
    };
    return { path, declaration };
};
const makeDomainsGenerator = (config) => (schema, outputAcc) => {
    const declarations = schema.domains?.map(makeMapper(config)) ?? [];
    return declarations.reduce((acc, elem) => {
        if (elem === undefined)
            return acc;
        const { path, declaration } = elem;
        const existing = acc[path];
        if (existing) {
            existing.declarations.push(declaration);
        }
        else {
            acc[path] = {
                declarations: [declaration],
            };
        }
        return acc;
    }, outputAcc);
};
exports.default = makeDomainsGenerator;
