"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tagged_comment_parser_1 = require("tagged-comment-parser");
const makeMapper = (style, config) => (enumDetails) => {
    // If an enum has a @type tag in the comment,
    // we will use that type instead of a generated one.
    const { tags } = (0, tagged_comment_parser_1.tryParse)(enumDetails.comment);
    if (tags?.type) {
        return undefined;
    }
    const { name, comment, path } = config.getMetadata(enumDetails, undefined, config);
    if (style === "type") {
        const declaration = {
            declarationType: "typeDeclaration",
            name,
            comment,
            exportAs: "default",
            typeDefinition: [
                "", // Start definition on new line
                ...enumDetails.values.map((value) => `| '${value}'`),
            ],
        };
        return { path, declaration };
    }
    else {
        const declaration = {
            declarationType: "enum",
            comment,
            name,
            exportAs: "default",
            values: enumDetails.values,
        };
        return { path, declaration };
    }
};
const makeEnumsGenerator = (config) => (schema, outputAcc) => {
    const declarations = schema.enums?.map(makeMapper(config.enumStyle, config)) ?? [];
    return declarations.reduce((acc, elem) => {
        if (elem === undefined)
            return acc;
        const { path, declaration } = elem;
        const existing = acc[path];
        if (existing) {
            existing.declarations.push(declaration);
        }
        else {
            acc[path] = {
                declarations: [declaration],
            };
        }
        return acc;
    }, outputAcc);
};
exports.default = makeEnumsGenerator;
