"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tagged_comment_parser_1 = require("tagged-comment-parser");
const makeMapper = (config) => (rangeDetails) => {
    // If a range has a @type tag in the comment,
    // we will use that type instead of a generated one.
    const { tags } = (0, tagged_comment_parser_1.tryParse)(rangeDetails.comment);
    if (tags?.type) {
        return undefined;
    }
    const { name, comment, path } = config.getMetadata(rangeDetails, undefined, config);
    // let rType: string;
    // const typeImports: TypeImport[] = [];
    // const mapped: TypeDefinition = config.typeMap[rangeDetails.innerType];
    // if (!mapped) {
    //   rType = 'unknown';
    //   console.warn(
    //     `Range '${name}' has unknown type '${rangeDetails.innerType}'`
    //   );
    // } else if (typeof mapped === 'string') {
    //   rType = mapped;
    // } else {
    //   rType = mapped.name;
    //   typeImports.push(...mapped.typeImports);
    // }
    const declaration = {
        declarationType: "typeDeclaration",
        name,
        comment,
        exportAs: "default",
        // typeDefinition: [`[lowerBound: ${rType}, upperBound: ${rType}]`],
        typeDefinition: ["string"],
        typeImports: [],
    };
    return { path, declaration };
};
const makeRangesGenerator = (config) => (schema, outputAcc) => {
    const declarations = schema.ranges?.map(makeMapper(config)) ?? [];
    return declarations.reduce((acc, elem) => {
        if (elem === undefined)
            return acc;
        const { path, declaration } = elem;
        const existing = acc[path];
        if (existing) {
            existing.declarations.push(declaration);
        }
        else {
            acc[path] = {
                declarations: [declaration],
            };
        }
        return acc;
    }, outputAcc);
};
exports.default = makeRangesGenerator;
