"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeGenerateIndexFile = void 0;
const path_1 = require("path");
function stringifyExportItem(item) {
    const prefix = item.exportAsType ? "type " : "";
    return `${prefix}${item.wasExportedAs === "default" ? "default as " : ""}${item.name}`;
}
const makeGenerateIndexFile = (config) => (outputAcc, instantiatedConfig) => {
    const allExports = {};
    for (const path of Object.keys(outputAcc)) {
        const file = outputAcc[path];
        allExports[path] = [];
        for (const declaration of file.declarations) {
            if (declaration.declarationType === "generic") {
                continue;
            }
            // eslint-disable-next-line unicorn/no-array-callback-reference
            if (config.filter && !config.filter(declaration)) {
                continue;
            }
            const { name, exportAs, declarationType } = declaration;
            allExports[path].push({
                name,
                wasExportedAs: exportAs,
                exportAsType: ["typeDeclaration", "interface"].includes(declarationType),
            });
        }
    }
    const lines = Object.keys(allExports).map((path) => {
        const exports = allExports[path];
        if (exports.length === 0) {
            return "";
        }
        const relativePath = (0, path_1.relative)(instantiatedConfig.outputPath, path);
        const line = `export { ${
        // eslint-disable-next-line unicorn/no-array-callback-reference
        exports.map(stringifyExportItem).join(", ")} } from './${relativePath}';`;
        return line;
    });
    const indexFile = {
        declarations: [
            {
                declarationType: "generic",
                lines,
            },
        ],
    };
    const path = (0, path_1.join)(instantiatedConfig.outputPath, "index");
    return {
        ...outputAcc,
        [path]: indexFile,
    };
};
exports.makeGenerateIndexFile = makeGenerateIndexFile;
const generateIndexFile = (0, exports.makeGenerateIndexFile)({});
exports.default = generateIndexFile;
